"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _harnoKernel = _interopRequireDefault(require("harno-kernel"));

var _request = _interopRequireDefault(require("harno-kernel/dist/services/node/request/request.service"));

var _errorHandler = _interopRequireDefault(require("harno-kernel/dist/services/node/errorHandler/error.handler.service"));

var _logger = _interopRequireDefault(require("harno-kernel/dist/services/node/logger/logger.service"));

var _config = _interopRequireDefault(require("harno-kernel/dist/services/shared/config/config.service"));

var _default = _interopRequireDefault(require("./tasks/default"));

var _code = _interopRequireDefault(require("./tasks/code.style"));

var _unit = _interopRequireDefault(require("./tasks/unit.tests"));

var _compile = _interopRequireDefault(require("./tasks/compile"));

var _jsdoc = _interopRequireDefault(require("./tasks/jsdoc"));

var _unitTests = _interopRequireDefault(require("./tasks/unit.tests.coverage"));

var _swagger = _interopRequireDefault(require("./tasks/swagger"));

var _files = _interopRequireDefault(require("./tasks/files.management"));

var _dist = _interopRequireDefault(require("./tasks/dist"));

var _explore = _interopRequireDefault(require("./tasks/explore"));

var _runner = _interopRequireDefault(require("./tasks/runner"));

/**
 * Class that contains different tools that will help you to build your application
 */
var Build =
/*#__PURE__*/
function () {
  /**
   * Registers all services and sets up config
   * @param {Object} config user defined build configuration
   */
  function Build(config) {
    (0, _classCallCheck2.default)(this, Build);
    this.kernel = new _harnoKernel.default();
    this.s = this.kernel.s;
    this.kernel.registerService('config', _config.default);
    this.kernel.registerService('logger', _logger.default);
    this.kernel.registerService('errorHandler', _errorHandler.default);
    this.kernel.registerService('request', _request.default);
    this.kernel.registerService('default', _default.default);
    this.kernel.registerService('codeStyle', _code.default);
    this.kernel.registerService('unitTests', _unit.default);
    this.kernel.registerService('compile', _compile.default);
    this.kernel.registerService('jsdoc', _jsdoc.default);
    this.kernel.registerService('unitTestsCoverage', _unitTests.default);
    this.kernel.registerService('swagger', _swagger.default); // conflicts with kernel swagger service (will be fixed in the future)

    this.kernel.registerService('fm', _files.default);
    this.kernel.registerService('explore', _explore.default);
    this.kernel.registerService('dist', _dist.default);
    this.kernel.registerService('runner', _runner.default);
    this.kernel.initDependencies({
      request: ['errorHandler'],
      errorHandler: ['logger'],
      codeStyle: ['config', 'logger'],
      unitTests: ['config', 'logger'],
      compile: ['config', 'logger'],
      jsdoc: ['config', 'logger'],
      unitTestsCoverage: ['config', 'logger'],
      swagger: ['config', 'request', 'logger'],
      explore: ['config', 'logger'],
      dist: ['config', 'fm', 'compile', 'logger'],
      runner: ['config', 'dist', 'compile', 'codeStyle', 'unitTests', 'unitTestsCoverage', 'jsdoc', 'swagger', 'logger']
    });

    var defaultConfigFile = _fs.default.readFileSync(_path.default.resolve(__dirname, './config.defaults.yml'));

    this.s.config.setConfig(config);
    this.s.config.setDefaultsYML(defaultConfigFile);
    this.c = this.s.config.c;
    this.s.logger.init(this.c.logger);
    this.s.errorHandler.init();
    this.s.dist.init();
    this.s.codeStyle.init();
    this.s.explore.init();
    this.s.jsdoc.init();
    this.s.swagger.init();
    this.s.unitTestsCoverage.init();
  }
  /**
   * Inits all gulp tasks
   * @param {Object} gulp gulp
   */


  (0, _createClass2.default)(Build, [{
    key: "initGulpTasks",
    value: function initGulpTasks(gulp) {
      var _this = this;

      this._gulp = gulp;
      this.s.unitTests.init(gulp);
      this.s.compile.init(gulp);
      this.s.runner.init(gulp);

      this._registerGulpTasks(['default'], this.s.default.default, {
        types: ['plain']
      });

      this._registerGulpTasks(['code-style', 'cs'], this.s.codeStyle.codeStyle, {
        watchFiles: this.c.codeStyle.watchFiles
      });

      this._registerGulpTasks(['unit-tests', 'ut'], this.s.unitTests.unitTests, {
        watchFiles: this.c.unitTests.watchFiles
      });

      this._registerGulpTasks(['compile', 'c'], this.s.compile.compile, {
        watchFiles: this.c.compile.watchFiles
      });

      this._registerGulpTasks(['jsdoc', 'j'], this.s.jsdoc.jsdoc, {
        watchFiles: this.c.jsdoc.watchFiles
      });

      this._registerGulpTasks(['unit-tests-coverage', 'utc'], this.s.unitTestsCoverage.unitTestsCoverage, {
        watchFiles: this.c.unitTestsCoverage.watchFiles
      });

      this._registerGulpTasks(['swagger', 'sg'], this.s.swagger.swagger, {
        watchFiles: this.c.swagger.watchFiles
      });

      this._registerGulpTasks(['explore', 'e'], this.s.explore.explore, {
        types: ['plain']
      });

      this._registerGulpTasks(['dist', 'd'], this.s.dist.dist, {
        watchFiles: this.c.dist.watchFiles
      });

      this._registerGulpTasks(['runner', 'r'], this.s.runner.runner, {
        watchFiles: this.c.dist.watchFiles
      });

      gulp.task('clean',
      /*#__PURE__*/
      (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee() {
        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _this.s.fm.remove(_this.c.clean.paths);

              case 1:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      })));
      gulp.task('test', gulp.series('clean', 'code-style', 'jsdoc', 'unit-tests', 'unit-tests-coverage'));
    }
    /**
     * Registers gulp tasks such as just plain task, fail-safe, watchers
     * @param {Array<string>} names task name
     * @param {Function(): boolean} task returns true if task succeeded
     * @param {Object} options register options
     * @param {string} [options.watchFiles=='all files'] files to watch for watcher
     * @param {Array<string>} [options.types=['plain','fail-safe','watcher']] defines types of gulp tasks, that should be created
     */

  }, {
    key: "_registerGulpTasks",
    value: function _registerGulpTasks(names, task, _ref2) {
      var _this2 = this;

      var _ref2$watchFiles = _ref2.watchFiles,
          watchFiles = _ref2$watchFiles === void 0 ? '**/*' : _ref2$watchFiles,
          _ref2$types = _ref2.types,
          types = _ref2$types === void 0 ? ['plain', 'fail-safe', 'watcher'] : _ref2$types;

      var mainTask =
      /*#__PURE__*/
      function () {
        var _ref3 = (0, _asyncToGenerator2.default)(
        /*#__PURE__*/
        _regenerator.default.mark(function _callee2() {
          var passed;
          return _regenerator.default.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  _context2.next = 2;
                  return task();

                case 2:
                  passed = _context2.sent;

                  if (passed) {
                    _context2.next = 5;
                    break;
                  }

                  throw new Error('Task failed');

                case 5:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2, this);
        }));

        return function mainTask() {
          return _ref3.apply(this, arguments);
        };
      }();

      if (types.includes('plain')) {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = names[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var name = _step.value;

            this._gulp.task("".concat(name), mainTask);
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }

      if (types.includes('fail-safe')) {
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = names[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var _name = _step2.value;

            this._gulp.task("".concat(_name, "-fail-safe"), task);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }

      if (types.includes('watcher')) {
        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = names[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var _name2 = _step3.value;

            this._gulp.task("".concat(_name2, "-watch"),
            /*#__PURE__*/
            (0, _asyncToGenerator2.default)(
            /*#__PURE__*/
            _regenerator.default.mark(function _callee3() {
              return _regenerator.default.wrap(function _callee3$(_context3) {
                while (1) {
                  switch (_context3.prev = _context3.next) {
                    case 0:
                      console.log("Watching: ".concat(watchFiles));
                      _context3.next = 3;
                      return task();

                    case 3:
                      _this2._gulp.watch(watchFiles, task);

                    case 4:
                    case "end":
                      return _context3.stop();
                  }
                }
              }, _callee3, this);
            })));
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
              _iterator3.return();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }
      }
    }
  }]);
  return Build;
}();

exports.default = Build;