"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _fs = _interopRequireDefault(require("fs"));

var _chalk = _interopRequireDefault(require("chalk"));

var _esdoc = _interopRequireDefault(require("esdoc"));

/**
 * Class that defines jsdoc tasks
 */
var Jsdoc =
/*#__PURE__*/
function () {
  function Jsdoc() {
    (0, _classCallCheck2.default)(this, Jsdoc);
  }

  (0, _createClass2.default)(Jsdoc, [{
    key: "init",

    /**
     * Inits service
     */
    value: function init() {
      this._config = this._d.config.c.jsdoc;
      this._logger = this._d.logger.createLogger('JsdocTask');
    }
    /**
     * Returns jsdoc task function
     * @return {Function} jsdoc task function
     */

  }, {
    key: "_jsdoc",

    /**
     * Generates js docs and tests them
     * @return {Promise<bool>} true if jsdoc passed tests
     */
    value: function () {
      var _jsdoc2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee() {
        var coverage, lint, threshold;
        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                this._logger.trace('Jsdoc task started');

                _esdoc.default.generate({
                  source: this._config.src,
                  destination: this._config.dest,
                  plugins: [{
                    name: 'esdoc-ecmascript-proposal-plugin',
                    option: {
                      objectRestSpread: true
                    }
                  }, {
                    name: 'esdoc-standard-plugin',
                    option: {
                      test: {
                        source: this._config.src
                      },
                      coverage: {
                        kind: ['class', 'method', 'get', 'set', 'constructor', 'function']
                      }
                    }
                  }]
                });

                coverage = this._getJsdocCoverage();
                lint = this._getJsdocLint();
                threshold = this._config.threshold;

                if (!(coverage < threshold)) {
                  _context.next = 8;
                  break;
                }

                this._logger.error(_chalk.default.red("Bad JS docs coverage: ".concat(coverage, "%. Threshold: ").concat(threshold, "%")));

                return _context.abrupt("return", false);

              case 8:
                if (!(lint.length > 0)) {
                  _context.next = 11;
                  break;
                }

                this._logger.error(_chalk.default.red("JS docs contains errors. Please fix them"));

                return _context.abrupt("return", false);

              case 11:
                this._logger.info(_chalk.default.green("JS docs coverage: ".concat(coverage, "%. Threshold: ").concat(threshold, "%")));

                return _context.abrupt("return", true);

              case 13:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function _jsdoc() {
        return _jsdoc2.apply(this, arguments);
      }

      return _jsdoc;
    }()
    /**
     * Gets js docs coverage
     * @return {number} js docs coverage
     */

  }, {
    key: "_getJsdocCoverage",
    value: function _getJsdocCoverage() {
      var jsdocCoverage = JSON.parse(_fs.default.readFileSync("".concat(this._config.dest, "coverage.json")));
      return parseFloat(jsdocCoverage.coverage);
    }
    /**
    * Gets js docs lint errors
    * @return {array} array of lint errors
    */

  }, {
    key: "_getJsdocLint",
    value: function _getJsdocLint() {
      return JSON.parse(_fs.default.readFileSync("".concat(this._config.dest, "lint.json")));
    }
  }, {
    key: "jsdoc",
    get: function get() {
      return this._jsdoc.bind(this);
    }
  }]);
  return Jsdoc;
}();

exports.default = Jsdoc;