"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _gulpConnect = _interopRequireDefault(require("gulp-connect"));

var _lazypipe = _interopRequireDefault(require("lazypipe"));

var _child_process = require("child_process");

var _chalk = _interopRequireDefault(require("chalk"));

var _path = _interopRequireDefault(require("path"));

/**
 * Class that defines runner tasks
 */
var Runner =
/*#__PURE__*/
function () {
  function Runner() {
    (0, _classCallCheck2.default)(this, Runner);
  }

  (0, _createClass2.default)(Runner, [{
    key: "init",

    /**
     * Inits runner service
     * @param {Object} gulp gulp instance
     */
    value: function init(gulp) {
      this._gulp = gulp;
      this._config = this._d.config.c.runner;
      if (typeof this._config.steps === 'string') this._config.steps = this._config.presets[this._config.steps] || [];
      this._logger = this._d.logger.createLogger('RunnerTask');
      this._childsToKill = [];
      this._webServers = {};
    }
    /**
     * Returns runner task function
     * @return {Function} runner task function
     */

  }, {
    key: "_runner",

    /**
     * Runs the steps in the config. From commands to livereload
     * @return {Promise<bool>} resolves with true when steps are finished
     */
    value: function () {
      var _runner2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee() {
        var _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _step$value, index, step, cwd;

        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                this._logger.trace('Runner task started');

                this._killAllChilds();

                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context.prev = 5;
                _iterator = this._config.steps.entries()[Symbol.iterator]();

              case 7:
                if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                  _context.next = 31;
                  break;
                }

                _step$value = (0, _slicedToArray2.default)(_step.value, 2), index = _step$value[0], step = _step$value[1];
                cwd = _path.default.resolve(step.root || './');

                this._logger.trace("Step ".concat(_chalk.default.yellow(index), " started ~ ").concat(_chalk.default.magenta(step.name), " ~ ").concat(cwd));

                _context.prev = 11;

                if (!step.tasks) {
                  _context.next = 15;
                  break;
                }

                _context.next = 15;
                return this._executeTasks(step.tasks, {
                  cwd: cwd
                });

              case 15:
                if (!step.exec) {
                  _context.next = 18;
                  break;
                }

                _context.next = 18;
                return this._executeCommand(step.exec.command, step.exec.args, {
                  wait: step.exec.wait,
                  cwd: cwd
                });

              case 18:
                if (!step.web) {
                  _context.next = 21;
                  break;
                }

                _context.next = 21;
                return this._webLivereload(step.web.host, step.web.port, {
                  cwd: cwd
                });

              case 21:
                this._logger.trace("Step ".concat(_chalk.default.yellow(index), " ").concat(_chalk.default.green('finished'), " ~ ").concat(_chalk.default.magenta(step.name)));

                _context.next = 28;
                break;

              case 24:
                _context.prev = 24;
                _context.t0 = _context["catch"](11);

                this._logger.error("Step ".concat(_chalk.default.yellow(index), " ").concat(_chalk.default.red('failed with an error'), " ~ ").concat(_chalk.default.magenta(step.name)), _context.t0);

                return _context.abrupt("return", false);

              case 28:
                _iteratorNormalCompletion = true;
                _context.next = 7;
                break;

              case 31:
                _context.next = 37;
                break;

              case 33:
                _context.prev = 33;
                _context.t1 = _context["catch"](5);
                _didIteratorError = true;
                _iteratorError = _context.t1;

              case 37:
                _context.prev = 37;
                _context.prev = 38;

                if (!_iteratorNormalCompletion && _iterator.return != null) {
                  _iterator.return();
                }

              case 40:
                _context.prev = 40;

                if (!_didIteratorError) {
                  _context.next = 43;
                  break;
                }

                throw _iteratorError;

              case 43:
                return _context.finish(40);

              case 44:
                return _context.finish(37);

              case 45:
                this._logger.info('Runner task finished successfully');

                return _context.abrupt("return", true);

              case 47:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[5, 33, 37, 45], [11, 24], [38,, 40, 44]]);
      }));

      function _runner() {
        return _runner2.apply(this, arguments);
      }

      return _runner;
    }()
    /**
     * Executes other gulp tasks
     * @param {Array<string>} tasks array of gulp task names
     * @param {Object} options options
     * @param {string} options.cwd current working directory in which task should be launched
     * @return {Promise} resolves when all tasks are finished
     */

  }, {
    key: "_executeTasks",
    value: function () {
      var _executeTasks2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee2(tasks, _ref) {
        var cwd, currentCwd, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, task, taskSuccedded;

        return _regenerator.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                cwd = _ref.cwd;
                currentCwd = process.cwd();
                process.chdir(cwd);
                _iteratorNormalCompletion2 = true;
                _didIteratorError2 = false;
                _iteratorError2 = undefined;
                _context2.prev = 6;
                _iterator2 = tasks[Symbol.iterator]();

              case 8:
                if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
                  _context2.next = 64;
                  break;
                }

                task = _step2.value;

                this._logger.trace("Task '".concat(task, "' started"));

                taskSuccedded = void 0;

                if (!(task === 'dist')) {
                  _context2.next = 18;
                  break;
                }

                _context2.next = 15;
                return this._d.dist.dist();

              case 15:
                taskSuccedded = _context2.sent;
                _context2.next = 56;
                break;

              case 18:
                if (!(task === 'compile')) {
                  _context2.next = 24;
                  break;
                }

                _context2.next = 21;
                return this._d.compile.compile();

              case 21:
                taskSuccedded = _context2.sent;
                _context2.next = 56;
                break;

              case 24:
                if (!(task === 'jsdoc')) {
                  _context2.next = 30;
                  break;
                }

                _context2.next = 27;
                return this._d.jsdoc.jsdoc();

              case 27:
                taskSuccedded = _context2.sent;
                _context2.next = 56;
                break;

              case 30:
                if (!(task === 'code-style')) {
                  _context2.next = 36;
                  break;
                }

                _context2.next = 33;
                return this._d.codeStyle.codeStyle();

              case 33:
                taskSuccedded = _context2.sent;
                _context2.next = 56;
                break;

              case 36:
                if (!(task === 'unit-tests')) {
                  _context2.next = 42;
                  break;
                }

                _context2.next = 39;
                return this._d.unitTests.unitTests();

              case 39:
                taskSuccedded = _context2.sent;
                _context2.next = 56;
                break;

              case 42:
                if (!(task === 'unit-tests-coverage')) {
                  _context2.next = 48;
                  break;
                }

                _context2.next = 45;
                return this._d.unitTestsCoverage.unitTestsCoverage();

              case 45:
                taskSuccedded = _context2.sent;
                _context2.next = 56;
                break;

              case 48:
                if (!(task === 'swagger')) {
                  _context2.next = 54;
                  break;
                }

                _context2.next = 51;
                return this._d.swagger.swagger();

              case 51:
                taskSuccedded = _context2.sent;
                _context2.next = 56;
                break;

              case 54:
                this._logger.error("Runner doesn't support '".concat(task, "' task, ignoring"));

                taskSuccedded = false;

              case 56:
                if (!taskSuccedded) {
                  _context2.next = 60;
                  break;
                }

                this._logger.info("Task '".concat(task, "' finished"));

                _context2.next = 61;
                break;

              case 60:
                throw new Error("Task '".concat(task, "' failed"));

              case 61:
                _iteratorNormalCompletion2 = true;
                _context2.next = 8;
                break;

              case 64:
                _context2.next = 70;
                break;

              case 66:
                _context2.prev = 66;
                _context2.t0 = _context2["catch"](6);
                _didIteratorError2 = true;
                _iteratorError2 = _context2.t0;

              case 70:
                _context2.prev = 70;
                _context2.prev = 71;

                if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
                  _iterator2.return();
                }

              case 73:
                _context2.prev = 73;

                if (!_didIteratorError2) {
                  _context2.next = 76;
                  break;
                }

                throw _iteratorError2;

              case 76:
                return _context2.finish(73);

              case 77:
                return _context2.finish(70);

              case 78:
                process.chdir(currentCwd);

              case 79:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this, [[6, 66, 70, 78], [71,, 73, 77]]);
      }));

      function _executeTasks(_x, _x2) {
        return _executeTasks2.apply(this, arguments);
      }

      return _executeTasks;
    }()
    /**
     * Executes any command
     * @param {string} command command
     * @param {Array<string>} args command arguments
     * @param {Object} options options
     * @param {string} options.cwd current working directory in which command should be executed
     * @param {string} options.wait defines if runner need to wait for this command to finish
     * @return {Promise} resolves when command is finished or right away if options.wait is false
     */

  }, {
    key: "_executeCommand",
    value: function () {
      var _executeCommand2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee3(command, args, _ref2) {
        var _this = this;

        var wait, cwd;
        return _regenerator.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                wait = _ref2.wait, cwd = _ref2.cwd;
                _context3.next = 3;
                return new Promise(function (resolve, reject) {
                  _this._logger.trace("Command '".concat(command, " ").concat(args.join(' '), "' executed. Cwd: ").concat(cwd));

                  var child = (0, _child_process.spawn)(command, args, {
                    cwd: cwd
                  });
                  child.stdout.pipe(process.stdout);
                  child.stderr.pipe(process.stderr);
                  child.on('error', function (err) {
                    _this._logger.error("Command '".concat(command, " ").concat(args.join(' '), "' throws an error"), err);

                    reject(err);
                  });
                  child.on('close', function (code) {
                    if (code === null) {
                      _this._logger.trace("Command '".concat(command, " ").concat(args.join(' '), "' terminated"));
                    } else if (code === 0) {
                      _this._logger.info(_chalk.default.green("Command '".concat(command, " ").concat(args.join(' '), "' finished")));
                    } else {
                      _this._logger.error(_chalk.default.red("Command '".concat(command, " ").concat(args.join(' '), "' exited with code: ").concat(code)));
                    }

                    resolve();
                  });

                  if (!wait) {
                    _this._childsToKill.push(child);

                    _this._logger.info("Command '".concat(command, " ").concat(args.join(' '), "' executed, but will not be waited to finish"));

                    resolve();
                  }
                });

              case 3:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function _executeCommand(_x3, _x4, _x5) {
        return _executeCommand2.apply(this, arguments);
      }

      return _executeCommand;
    }()
    /**
     * Kills all child processes which were not killed or finished previously
     */

  }, {
    key: "_killAllChilds",
    value: function _killAllChilds() {
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = this._childsToKill[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var child = _step3.value;
          child.kill();
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      this._childsToKill = [];

      this._logger.trace("All commands, previously started, killed");
    }
    /**
     * Creates server or reloads it if it is created
     * @param {string} host server host
     * @param {number} port server port
     * @param {Object} options options
     * @param {string} options.cwd current working directory in which command should be executed
     * @return {Promise} resolves when server is created or reloaded
     */

  }, {
    key: "_webLivereload",
    value: function () {
      var _webLivereload2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee4(host, port, _ref3) {
        var _this2 = this;

        var cwd, currentCwd, reload;
        return _regenerator.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                cwd = _ref3.cwd;

                if (this._webServers["".concat(host, ":").concat(port)]) {
                  _context4.next = 9;
                  break;
                }

                currentCwd = process.cwd();
                process.chdir(cwd);

                this._logger.info("Server on ".concat(host, ":").concat(port, " started"));

                this._webServers["".concat(host, ":").concat(port)] = _gulpConnect.default.server({
                  root: './',
                  port: port,
                  host: host,
                  livereload: true
                });
                process.chdir(currentCwd);
                _context4.next = 13;
                break;

              case 9:
                reload = (0, _lazypipe.default)().pipe(function () {
                  return _gulpConnect.default.reload();
                });
                _context4.next = 12;
                return new Promise(function (resolve, reject) {
                  _this2._gulp.src(cwd).pipe(reload()).on('error', reject).on('end', resolve);
                });

              case 12:
                this._logger.info("Server on ".concat(host, ":").concat(port, " reloaded"));

              case 13:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function _webLivereload(_x6, _x7, _x8) {
        return _webLivereload2.apply(this, arguments);
      }

      return _webLivereload;
    }()
  }, {
    key: "runner",
    get: function get() {
      return this._runner.bind(this);
    }
  }]);
  return Runner;
}();

exports.default = Runner;