"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _swaggerParser = _interopRequireDefault(require("swagger-parser"));

var _child_process = require("child_process");

/**
 * Class that defines swagger tasks
 */
var Swagger =
/*#__PURE__*/
function () {
  function Swagger() {
    (0, _classCallCheck2.default)(this, Swagger);
  }

  (0, _createClass2.default)(Swagger, [{
    key: "init",

    /**
     * Inits service
     */
    value: function init() {
      this._config = this._d.config.c.swagger;
      this._logger = this._d.logger.createLogger('SwaggerTask');
    }
    /**
     * Returns swagger task function
     * @return {Function} swagger task function
     */

  }, {
    key: "_swagger",

    /**
     * Validates swagger docs
     * @return {Promise<bool>} true if swagger passed tests
     */
    value: function () {
      var _swagger2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee() {
        var docs;
        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                this._logger.trace('Swagger task started');

                _context.prev = 1;
                _context.next = 4;
                return this._getDocs();

              case 4:
                docs = _context.sent;
                _context.next = 7;
                return this._validateDocs(docs);

              case 7:
                return _context.abrupt("return", true);

              case 10:
                _context.prev = 10;
                _context.t0 = _context["catch"](1);

                this._logger.error('Swagger validation failed with an error:', _context.t0);

                return _context.abrupt("return", false);

              case 14:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[1, 10]]);
      }));

      function _swagger() {
        return _swagger2.apply(this, arguments);
      }

      return _swagger;
    }()
    /**
     * Gets swagger docs from running server
     * @return {Promise<Object>} resolves with swagger docs
     */

  }, {
    key: "_getDocs",
    value: function () {
      var _getDocs2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee2() {
        var _this = this;

        var docs, app, responce;
        return _regenerator.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                app = (0, _child_process.spawn)(this._config.startServer.command, this._config.startServer.args, this._config.startServer.options);
                _context2.prev = 1;
                app.stdout.pipe(process.stdout);
                app.stderr.pipe(process.stderr);
                _context2.next = 6;
                return new Promise(function (resolve) {
                  return setTimeout(resolve, _this._config.delay);
                });

              case 6:
                _context2.next = 8;
                return this._d.request.request(this._config.url);

              case 8:
                responce = _context2.sent;
                docs = JSON.parse(responce);
                app.kill();
                _context2.next = 13;
                return new Promise(function (resolve, reject) {
                  app.on('error', function (err) {
                    reject(err);
                  });
                  app.on('close', function (code) {
                    resolve();
                  });
                });

              case 13:
                return _context2.abrupt("return", docs);

              case 16:
                _context2.prev = 16;
                _context2.t0 = _context2["catch"](1);
                app.kill();
                throw _context2.t0;

              case 20:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this, [[1, 16]]);
      }));

      function _getDocs() {
        return _getDocs2.apply(this, arguments);
      }

      return _getDocs;
    }()
    /**
     * Validates swagger docs
     * @param {Object} docs swagger docs
     * @return {Promise} resolves when validation finishes
     * @throw {Error} in case of validation error
     */

  }, {
    key: "_validateDocs",
    value: function () {
      var _validateDocs2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee3(docs) {
        var paths, schemas, pathsStrings, _i, path, methods;

        return _regenerator.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (docs) {
                  _context3.next = 2;
                  break;
                }

                throw new Error('Swagger docs not found, mb server wasn\'t started or some configuration problem');

              case 2:
                _context3.next = 4;
                return _swaggerParser.default.validate(docs);

              case 4:
                paths = Object.keys(docs.paths);
                schemas = Object.keys(docs.definitions);
                pathsStrings = [];

                for (_i = 0; _i < paths.length; _i++) {
                  path = paths[_i];
                  methods = Object.keys(docs.paths[path]).join(', ').toUpperCase();
                  pathsStrings.push("".concat(methods, " ").concat(path));
                }

                this._logger.trace("Paths - ".concat(paths.length, ": ").concat(pathsStrings.join(' | ')));

                this._logger.trace("Schemas - ".concat(schemas.length, ": ").concat(schemas.join(' | ')));

                this._logger.info('Swagger task finished successfully');

              case 11:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function _validateDocs(_x) {
        return _validateDocs2.apply(this, arguments);
      }

      return _validateDocs;
    }()
  }, {
    key: "swagger",
    get: function get() {
      return this._swagger.bind(this);
    }
  }]);
  return Swagger;
}();

exports.default = Swagger;