"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _fs = _interopRequireDefault(require("fs"));

var _child_process = require("child_process");

/**
 * Class that defines unit tests coverage tasks
 */
var UnitTestsCoverage =
/*#__PURE__*/
function () {
  function UnitTestsCoverage() {
    (0, _classCallCheck2.default)(this, UnitTestsCoverage);
  }

  (0, _createClass2.default)(UnitTestsCoverage, [{
    key: "init",

    /**
     * Inits service
     */
    value: function init() {
      this._config = this._d.config.c.unitTestsCoverage;
      this._logger = this._d.logger.createLogger('UnitTestsCoverageTask');
    }
    /**
     * Returns unit tests coverage task function
     * @return {Function} unit tests coverage task function
     */

  }, {
    key: "_unitTestsCoverage",

    /**
     * Creates coverage report and checks thresholds
     * @return {Promise<bool>} resolves with true if thresholds passed
     */
    value: function () {
      var _unitTestsCoverage2 = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee() {
        var _this = this;

        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                this._logger.trace('Unit tests coverage task started');

                _context.next = 3;
                return new Promise(function (resolve, reject) {
                  var nyc = (0, _child_process.spawn)(_this._config.nycBin, ['--require', '@babel/register', '--reporter', 'html', '--reporter', 'json-summary', '--tempDir', "".concat(_this._config.dest, ".nyc_output"), '--reportDir', _this._config.dest, '--include', _this._config.jsFiles, _this._config.mochaBin, '-b', _this._config.testFiles, '--color']);
                  nyc.stdout.pipe(process.stdout);
                  nyc.stderr.pipe(process.stderr);
                  nyc.on('error', function (err) {
                    reject(err);
                  });
                  nyc.on('close', function (code) {
                    resolve();
                  });
                });

              case 3:
                return _context.abrupt("return", this._thresholdsSummary());

              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function _unitTestsCoverage() {
        return _unitTestsCoverage2.apply(this, arguments);
      }

      return _unitTestsCoverage;
    }()
    /**
     * Outputs coverage thresholds report
     * @return {bool} true if coverage thresholds passed
     */

  }, {
    key: "_thresholdsSummary",
    value: function _thresholdsSummary() {
      var coverage = JSON.parse(_fs.default.readFileSync("".concat(this._config.dest, "coverage-summary.json")));
      var thresholds = this._config.thresholds;
      var unitTestsCoveragePassed = true;
      var _arr = ['lines', 'branches', 'functions', 'statements'];

      for (var _i = 0; _i < _arr.length; _i++) {
        var coverageUnit = _arr[_i];
        var threshold = thresholds[coverageUnit];
        var current = coverage.total[coverageUnit].pct;
        if (current === 'Unknown') current = 0;

        if (current < threshold) {
          this._logger.error("Bad '".concat(coverageUnit, "' coverage: ").concat(current, "% (").concat(threshold, "%)"));

          unitTestsCoveragePassed = false;
        } else {
          this._logger.trace("Good '".concat(coverageUnit, "' coverage: ").concat(current, "% (").concat(threshold, "%)"));
        }
      }

      if (unitTestsCoveragePassed) {
        this._logger.info('Unit tests coverage task finished successfully');
      } else {
        this._logger.error('Unit tests coverage task failed');
      }

      return unitTestsCoveragePassed;
    }
  }, {
    key: "unitTestsCoverage",
    get: function get() {
      return this._unitTestsCoverage.bind(this);
    }
  }]);
  return UnitTestsCoverage;
}();

exports.default = UnitTestsCoverage;